/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 2.2 (Rejection-from-Uniform-Hat), p. 17 
 *
 * this file: algo02_02.c
 *
 * For a demo how to use this algorithm see the file "demo_algo02_02.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

double generator02_02(double (*pdf)(double), double bl, double br, double m)
/*
 * Algorithm 2.2 (rejection from uniform hat) p.17 
 *
 * pdf... function pointer to the density
 * bl ... left border of domain
 * br ... right border of domain
 * m  ... upper bound for the density.
 */
{ double u, v, x, y;

/*1*/  while(1){ 
/*2*/    u = uniform();
/*3*/    x = bl + (br-bl)*u;
/*4*/    v = uniform();
/*5*/    y = m*v;
/*6+7*/  if(y<=(*pdf)(x)) return x;
       }
}




