/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 9.5 (Decreasing Hazard Rate dynamic Thinning), p. 198 
 *
 * this file: algo09_05.c
 *
 * For a demo how to use this algorithm see the file "demo_algo09_05.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

double generator09_05(double (*h)(double))
/*
 * Algorithm 9.5 (Decreasing Hazard Rate dynamic Thinning), p. 198 
 *
 * h   ... function pointer to the bounded, decreasing hazard rate h(x)
 */
{ double x, u, v, e, lamda;
  int i;  

/*1*/ x = 0.;
      i = 1;
/*2*/ while(1){
/*3*/	lamda = h(x);
/*4*/	u = uniform();
/*5*/	e = -log(1.-u)/lamda;
/*6*/	x = x + e;
/*7*/	v = lamda*uniform();
/*8*/	if(v <= (*h)(x)) 
/*9*/     return x;
	i++;
      }
}



