/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 10.4 (Automatic-Rejection-Inversion), p. 226
 *
 * this file: algo10_04.c
 *
 * For a demo how to use this algorithm see the file "demo_algo10_04.c".
 *
 */

#include <math.h>
#include <stdio.h>
#include <malloc.h>
#include "uniform.h"
#include "algo.h"



#define C -0.5
#define T(x) (-1./sqrt(x))
#define F(x) (-1./(x))
#define FM(x) (-1./(x))


#define MAX(a,b) (((a)<(b))?(b):(a))
#define PARLET(a,b) do{a.d1=b.d1;a.d2=b.d2;a.d3=b.d3;a.d4=b.d4;a.i1=b.i1;a.i2=b.i2;a.i3=b.i3;}while(0)

#define P(x) (*(t->hh))(&(t->par),x)


TOTAL *setup10_04(double (*hh)(PARAM *param,long x),PARAM *param,long m,long la,long ra,double pma)
/*
 * Algorithm 10.4 (Automatic-Rejection-Inversion), p. 226
 *
 * the functioner returns a pointer to teh structure holding all necessary constants
 *
 * INPUTS:
 * hh....pointer to pmf, 
 *  param .... paramaters of pmf, 
 *  pma ... < 0 means pmf has sum 1; 
 *      pma>0 is the approximate mode probability as the sum of the pmf<>1
 *  m,la,ra...mode, smallest, largest number with probability>0
 *
 */
{ 
       TOTAL *t;
       double v[2],at[2];
       long int b[2],i,d;
       double t0;
       char setup=1;

       if (la>m||ra<m){ 
	 printf("!!!ERROR setup10_04(): mode or domain borders wrong!!\n");
	 t=NULL;
	 return(t);
       }
       t=malloc(sizeof(TOTAL));
       PARLET((t->par),(*param));
       t->sign[0]= -1;
       t->sign[1]=1;
       t->m=m;
       t->hh=hh;
/*2*/  b[0]=la;
       b[1]=ra;
       t->pm=P(m);
/*3*/  if(pma<=0.) /*this is the case where the exact probabilitis are known*/
	 d=MAX(2,(long int)( 0.66/t->pm));
       else /*the probabilities are only known up to a constant factor and pma*/
	 d=MAX(2,(long int)( 0.66/pma));/*is an estimate for the modal probability*/
       do{
/*4*/	 for(i=0;i<=1;i++){
/*5*/	   t->x[i]=m+t->sign[i]*d;
/*6*/	   if(t->sign[i]*t->x[i]+1>t->sign[i]*b[i]) {  
/*7*/	     v[i]=0; t->s[i]=b[i];}
/*8*/	   else{ 
/*9*/	     t->y[i]=T(P(t->x[i]));
	     t->ys[i]=t->sign[i]*(T(P(t->x[i]+t->sign[i]))-t->y[i]);
	     t->s[i]=(long int)(0.5+t->x[i]+(T(t->pm)-t->y[i])/t->ys[i]);
/*10*/	     t->Hat[i]=F(t->y[i]+t->ys[i]*(t->s[i]+t->sign[i]*1.5-t->x[i]))/
	       t->ys[i]-t->sign[i]*P(t->s[i]+t->sign[i]); 
/*11*/	     at[i]=t->x[i]+(FM(t->ys[i]*t->Hat[i])-t->y[i])/t->ys[i]; 
/*12*/	     v[i]= t->sign[i]*(F(t->y[i]+t->ys[i]*(b[i]+t->sign[i]*0.5-t->x[i]))/
			       t->ys[i]-F(t->y[i]+t->ys[i]*(at[i]-t->x[i]))/t->ys[i]);
	   }
/*13*/	   t->ac[i]=t->s[i]+t->sign[i]*(P(t->s[i])/t->pm-0.5);
	 }   
/*14*/	 t->vc=t->pm*(t->ac[1]-t->ac[0]); 
	 t->vt=t->vc+v[0]+v[1];
	 t->vcr=t->vc+v[1];
	 if (setup==1){
/*15*/	   t0=(C<0?1./(1.-pow((1./(1.+C)),(1+1./(C+1.e-10)))):1.582); 
	   if (pma>0.) t0*=pma/t->pm;
/*16*/	   if (t->vt<=t0) setup=0;
	   else{ 
/*17*/	     setup=2;
	     if(pma>0.) d=(long int)t0/t->pm;
	     else d=(long int)t0/pma;
	   }
	 }
	 else setup=0;
       }while(setup);
       if(t->vt>100*t0||(t->vt<.9&&pma<0.)||(t->vt<0.)){
	 printf("!!!ERROR setup10_04() could not construct a hat!!\n");
	 printf(" distribution not T-concave or some other error!!\n");
	 free(t);
	 t=NULL;
	 return(t);
       }
       return(t);
}


int generator10_04(TOTAL *t)
     /* calls the genrator */ 
{ 
      double U,X,h;
      long k,i;
      if(t==NULL){ 
	printf("!!ERROR generator10_04!! NULL-pointer was passed!\n");
	return(-9999);}
/*18*/while(1){ 
/*19*/	U=uniform()*t->vt;
/*20*/	if (U<=t->vc){ 
/*21*/	  X=U*(t->ac[1]-t->ac[0])/t->vc+t->ac[0]; 
/*22*/	  k=(long int)(X+0.5);
/*23*/	  if(k<t->m) i=0; 
	  else  i=1;
/*24*/	  h=0.5-P(k)/t->pm;
/*25*/	  if (h<=t->sign[i]*(k-X)) 
/*26*/      return k;
	}
/*27*/	else{ 
/*28*/	  if (U<= t->vcr){
/*29*/	    i=1; 
	    U=U-t->vc;
	  } 
/*30*/	  else{
/*31*/	    i= 0;
	    U=U-t->vcr;
	  }
/*32*/	  U=t->Hat[i]+t->sign[i]*U; 
	  X=t->x[i]+(FM(U*t->ys[i])-t->y[i])/t->ys[i];
	  k=(long int)(X+0.5);
/*33*/	  h=t->sign[i]*F(t->y[i]+t->ys[i]*(k+t->sign[i]*0.5-t->x[i]))/
	    t->ys[i]-P(k);
/*34*/	  if (t->sign[i]*U>=h) 
/*35*/      return k;
	}
      }
}

int free10_04(TOTAL *t)
     /* frees the allocated memory */
{  free(t);
 return 1;
}
#undef P














