/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 11.3 (Multinormal-Cholesky, p. 250)
 *
 * this file: demo_algo11_03.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo11_03 demo_algo11_03.c algo11_03.c algo11_04.c algo02_11.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"



#define CLAS 300

#define DIM 4

static long int obs[DIM+1][CLAS];

int main()
{ int i,j;
 double sum,x[DIM+1],varsum,sigmasum,sigma[DIM+1]; 
 double     S[(DIM+1)*(DIM+1)]={0., 0.,0. ,0. ,0.,
                                0.,7. ,0.9,0.2,0.1,
                                0.,0.9,1. ,0.4,0.3,
                                0.,0.2,0.4,3. ,0.4,
                                0.,0.1,0.3,0.4,1. };

 printf("For the below variance-covariance matrix we generrate multinormal vectors:\n");
 printmatrix(DIM,S);

 setup11_03(DIM,S);/* calls set-up*/

 printf("Demo for Algorithm 11.3\n");
 printf("first we display ten generated vectors\n");
 for(i=0;i<10;i++){
   generator11_03(x);
   for(j=1;j<DIM;j++) printf("%8.3f, ",x[j]);
   printf("%8.3f\n",x[DIM]);
 }

 setseed(13324821); /*set seed for uniform generator*/

 printf("Now we test all marginal distributions and the sum-distribution\n");
 printf("of the generated vectors\n");

 for(j=0;j<=DIM;j++)
   for(i=0;i<CLAS;i++) obs[j][i]=0;

 varsum=0.;
 for(j=1;j<=DIM;j++){
   sigma[j]=sqrt(S[j*(DIM+1)+j]);
   for(i=1;i<=DIM;i++) varsum+= S[j*(DIM+1)+i];
 }
 sigmasum = sqrt(varsum);
 /* printf("varsum %f sigmasum %f\n",varsum,sigmasum);*/


 for(i=0;i<CLAS*CLAS;i++){
   generator11_03(x);
   sum = 0.;
   for(j=1;j<=DIM;j++){
     sum+=x[j];
     obs[j][(int)(cdf_normal(x[j]/sigma[j])*CLAS)]++;
   }
   obs[0][(int)(cdf_normal(sum/sigmasum)*CLAS)]++;
 }

 for(j=0;j<=DIM;j++){
   if(j==0) printf("\nchi2-test for the sum of all components\n");
   else printf("\nchi2-test for component %d:\n",j);
   chi2test(obs[j],CLAS,CLAS*CLAS,1); 
 }
 
 return 0;
}

