/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 11.8 (Uniform-Ball, p. 255)
 *
 * this file: demo_algo11_08.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo11_08 demo_algo11_08.c algo11_08.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"

#ifndef M_PI
#define M_PI       3.14159265358979323846264338328      /* pi */
#endif

double cdfmarginal(double x)
/* this is the CDF of the marginal distribution for dimension 2.
 * it is only used for the test.
 */
{ return  (0.5*x*sqrt(1-x*x)+asin(x)*0.5+M_PI*0.25)*2/M_PI;
}

#define CLAS 500

static long int obs[2][CLAS];

int main()
{ int i,j,d;
 double x[10];

 setseed(13324821); /*set seed for uniform generator*/

 printf("Demo for Algorithm 11.8\n");
 printf("first we display ten generated vectors for dimension 4\n");
 d=4;
 for(i=0;i<10;i++){
   generator11_08(d,x);
   for(j=1;j<d;j++) printf("%8.3f, ",x[j]);
   printf("%8.3f\n",x[d]);
 }


 printf("Now we test the marginal distributions for dimension 2\n");

 for(i=0;i<CLAS;i++){
   obs[0][i]=0;
   obs[1][i]=0;
 }

 for(i=0;i<CLAS*CLAS;i++){
   generator11_08(2,x);
   obs[0][(int)(cdfmarginal(x[1])*CLAS)]++;
   obs[1][(int)(cdfmarginal(x[2])*CLAS)]++;
 }
 printf("\nFor the first component:\n");
 chi2test(obs[0],CLAS,CLAS*CLAS,1); 
 printf("\nFor the second component:\n");
 chi2test(obs[1],CLAS,CLAS*CLAS,1); 
 
 return 0;
}



   

