/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 12.6 (4 Moments continuous), p. 326 
 *
 * this file: demo_algo12_06.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo12_06 demo_algo12_06.c algo12_06.c algo12_05.c algo02_11.c uniform.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"






#define N 100000 /* number of repetitons */


int main()
{ int i,n,M=N;
 double x,an,an1,dx,dx2;
 double moments[5],mu1=0.,mu2=2.,mu3=3.,mu4=20., emu1,emu2,emu3,emu4;

 setseed(13324885); /*set seed for uniform generator*/

  moments[0]=1.;  /* dummy field */
  for (i=1; i<=4; i++)
    moments[i] = 0.;

  setup12_06(mu1,mu2,mu3,mu4);
  printf("\nDemo for Algorithm 12.6\n");
  printf("first we display ten numbers generated from the empirical distribution\n\n");
  for(i=0;i<10;i++){
    x = generator12_06();
    printf("   x %f\n",x);
  }

 printf("\nNow we make a check if the desired and empirical moments are close for a sample of size %d\n", N);

  for (n=1; n<=N; n++) 
    {      
      x=generator12_06();
      
      an  = (long double)n;
      an1 = an-1.;
      dx  = (x-moments[1]) / an;
      dx2 = dx*dx;
      
      moments[4] -= dx*(4.*moments[3]-dx*(6.*moments[2]+an1*(1.+an1*an1*an1)*dx2));
      moments[3] -= dx*(3.*moments[2]-an*an1*(an-2.)*dx2);
      moments[2] += an*an1*dx2;
      moments[1] += dx;      
    }
  
  /* compute central moments */
  /* moments[1] is already the first moment. no division necessary. */

  for (i=2; i<=4; i++)
    moments[i] /= M;

  /* compute moments about the origin */

  emu1=moments[1];
  emu2=moments[2]+(emu1*emu1);
  emu3=moments[3]+(3*emu1*emu2)-(2*emu1*emu1*emu1);
  emu4=moments[4]+(4*emu3*emu1)-(6*emu2*emu1*emu1)+(3*emu1*emu1*emu1*emu1);

  /* now print results */
  
  printf("\n   Desired Moments: ");
  printf("mu1= %6.3f  mu2= %6.3f  mu3= %6.3f  mu4= %6.3f \n",mu1,mu2,mu3,mu4);
  printf(" Estimated Moments: ");
  printf("mu1= %6.3f  mu2= %6.3f  mu3= %6.3f  mu4= %6.3f \n\n",emu1,emu2,emu3,emu4);
      
 return 0;
}
